
%clear workspace
close all;
clear all;

%% Definition of initial concentration & volumes & mol. mass & simulation time

NrSpecies = 4;

% Reaction volumes of each species in units [L]
V = zeros(NrSpecies,1);
V(1) = 1e2;% reaction volume of X1
V(2) = 1e2;% reaction volume of X2
V(3) = 1e1;% reaction volume of X3
V(4) = 1e3;% reaction volume of X4

X0 = zeros(NrSpecies,1);
% Initial conditions
% in [mole]
X0(1) = 1e3;% X1 Substrate 
X0(2) = 30;  % X2 Enzyme
X0(3) = 0;  % X3 Complex 
X0(4) = 0;  % X4 Product

x0 = X0./V;

% avogadro constant
Na = 6.022e23; % [molecules/mol]

%molecular weights [g/mol]
MW = zeros(NrSpecies,1);
MW(1) = 6;
MW(2) = 10;
MW(3) = MW(1)+MW(2);%complex
MW(4) = 6;

% Simulations time
Simtime = 100; %[min]
% accuracy of ODE solver
RTol = 1e-4; %relative tolerance
ATol = 1e-6; % absolute tolerance



%% Definition of model reactions (only constant constituents)

% Model reactions

%  r1: X1 + X2  -> X3       ; rate: kon
%  r2: X3       -> X1 + X2  ; rate: koff
%  r3: X3       -> X4 + X2  ; rate: kcat
global NrReactions
NrReactions = 3;

% Stoichiometric Matrix 
% unit: # of molecules/moles changed by reaction
global S
S = zeros(NrSpecies,NrReactions);
%          X1   X2      X3      X4
S(:,1) = [-1;   -1;     1;      0]; %r1
S(:,2) = [1;    1;      -1;     0]; %r2
S(:,3) = [0;    1;      -1;     1]; %r3

% reaction parameters 
global k
k = ones(NrReactions,1);
k(1) = 3e2; %kon; unit [L^2/mole*min]
k(2) = 5e-0; %koff; unit [L/min]
k(3) = 5e0;%kcat; unit [L/min]


%% Simulate
%settings of the ODE solver (mass balance equations, accuracy (relative, absolute))
options = odeset('Mass',diag(V),'RelTol',RTol,'AbsTol',ATol); 
[t,x] = ode15s(@RHS,[0 Simtime],x0,options);

figure(1)
plot(t,x)
xlabel('time [min]')
ylabel('concentrations [mol/L]')
legend('X1','X2','X3','X4')
PlotNicely(1)

figure(2)
plot(t,(x*V).*Na) % sum over # mole sum_i ([mole_i/L] * [L]) - > [mole]
xlabel('time [min]')
ylabel('total # molecules in system')
PlotNicely(2)

figure(3)
plot(t,x*(V.*MW))
xlabel('time [min]')
ylabel('total mass in system [g]')
PlotNicely(3)