
%clear workspace
close all;
clear all;

%% Definition of initial concentration & volumes & mol. mass & simulation time

NrSpecies = 3;

% Reaction volumes of each species in units [L]
V = zeros(NrSpecies,1);
V(1) = 1;% reaction volume dosing compartment (source)
V(2) = 74;% reaction volume of central compartment
V(3) = 1;% reaction volume of sink (Excretion/Elimination/Non-Absortion Compartment)

X0 = zeros(NrSpecies,1);
% Initial conditions
% in [mole]
X0(1) = 50/455;% Dose 
X0(2) = 0;  % central compartment/plasma 
X0(3) = 0;  % Excreted amount

x0 = X0./V;

% avogadro constant
Na = 6.022e23; % [molecules/mol]

%molecular weights [g/mol]
MW = zeros(NrSpecies,1);
MW(1) = 455;
MW(2) = 455;
MW(3) = 455;

% Simulations time
Simtime = 100; %[min]
% accuracy of ODE solver
RTol = 1e-4; %relative tolerance
ATol = 1e-6; % absolute tolerance



%% Definition of model reactions (only constant constituents)

% Model reactions

%  r1: X1  -> X2 ; rate: Fbio*ka
%  r2: X2  -> X3 ; rate: ke
global NrReactions
NrReactions = 3;

% Stoichiometric Matrix 
% unit: # of molecules/moles changed by reaction
global S
S = zeros(NrSpecies,NrReactions);
%          X1   X2      X3      
S(:,1) = [-1;   1;     0;]; %r1
S(:,2) = [-1;   0;     1;]; %r2
S(:,3) = [0;    -1;    1;]; %r3

% reaction parameters 
global k
Fbio = 0.4;% 40% of drug enters the body/is absorbed
k = ones(NrReactions,1);
k(1) = Fbio*0.12; %Fbio*ka; unit [L/min]
k(2) = (1-Fbio)*0.12; %(1-Fbio)*ka; unit [L/min]
k(3) = 0.15*V(2); %CL = Vss*ke; unit [L/min]


%% Simulate
%settings of the ODE solver (mass balance equations, accuracy (relative, absolute))
options = odeset('Mass',diag(V),'RelTol',RTol,'AbsTol',ATol); 
[t,x] = ode15s(@RHSPK,[0 Simtime],x0,options);

figure(1)
plot(t,x(:,2))
xlabel('time [min]')
ylabel('concentrations [mol/L]')
set(gca,'YScale','log')%plot y-axis logarithmically
legend('central cmp')
PlotNicely(1)

figure(2)
plot(t,x(:,[1 3]).*repmat(MW([1 3])',length(t),1))
xlabel('time [min]')
ylabel('[mg]')
set(gca,'YScale','log')%plot y-axis logarithmically
legend('dosing cmp.','sink')
PlotNicely(2)

figure(4)
plot(t,(x*V).*Na) % sum over # mole sum_i ([mole_i/L] * [L]) - > [mole]
xlabel('time [min]')
ylabel('total # molecules in system')
PlotNicely(4)

figure(5)
plot(t,x*(V.*MW))
xlabel('time [min]')
ylabel('total mass in system [g]')
PlotNicely(5)