
%clear workspace
close all;
clear all;

%% Definition of initial concentration & volumes & mol. mass & simulation time

NrSpecies = 3;

% Reaction volumes of each species in units [L]
V = zeros(NrSpecies,1);
V(1) = 1;% reaction volume dosing compartment (source)
V(2) = 74;% reaction volume of central compartment
V(3) = 1;% reaction volume of sink (Excretion/Elimination/Non-Absortion Compartment)

X0 = zeros(NrSpecies,1);
% Initial conditions
% in [mole]
dose = 50/455;
X0(1) = dose;% Dose 
X0(2) = 0;  % central compartment/plasma 
X0(3) = 0;  % Excreted amount

x0 = X0./V;

% avogadro constant
Na = 6.022e23; % [molecules/mol]

%molecular weights [g/mol]
MW = zeros(NrSpecies,1);
MW(1) = 455;
MW(2) = 455;
MW(3) = 455;

% Simulations time
Simtime = 60*24*4; %[min]
% accuracy of ODE solver
RTol = 1e-4; %relative tolerance
ATol = 1e-6; % absolute tolerance



%% Definition of model reactions (only constant constituents)

% Model reactions

%  r1: X1  -> X2 ; rate: Fbio*ka
%  r2: X2  -> X3 ; rate: ke
global NrReactions
NrReactions = 3;

% Stoichiometric Matrix 
% unit: # of molecules/moles changed by reaction
global S
S = zeros(NrSpecies,NrReactions);
%          X1   X2      X3      
S(:,1) = [-1;   1;     0;]; %r1
S(:,2) = [-1;   0;     1;]; %r2
S(:,3) = [0;    -1;    1;]; %r3

% reaction parameters 
global k
Fbio = 0.4;% 40% of drug enters the body/is absorbed
ka = 0.002;%[1/min]
k = ones(NrReactions,1);
k(1) = Fbio*ka; %Fbio*ka; unit [L/min]
k(2) = (1-Fbio)*ka; %(1-Fbio)*ka; unit [L/min]
k(3) = 0.002*V(2); %CL = Vss*ke; unit [L/min]


%% Simulate
%settings of the ODE solver (mass balance equations, accuracy (relative, absolute))
options = odeset('Mass',diag(V),'RelTol',RTol,'AbsTol',ATol); 

OutInt = 1;%output interval; written every ... min
tOutput = 0:OutInt:Simtime;%time points
%initialize Trajectory
xOutput = zeros(length(tOutput),NrSpecies);
DosingInterval = 12*60; %every ... minutes

actualtime = 0;%actual runtimne
Dose = 1;%Dosing event number ... 
RunIdxStart = 1;%running start index for storage

while actualtime < Simtime %break condition
    %output time points of interest
    tspan = actualtime:OutInt:Dose*DosingInterval; 
    %running end index for storage
    RunIdxEnd = RunIdxStart + length(tspan) - 1;
    %simulation over one dosing interval
    [t,x] = ode15s(@RHSPK,tspan,x0,options);
    %store trajectory
    xOutput(RunIdxStart:RunIdxEnd,:) = x;
    %update running index for storage
    RunIdxStart = RunIdxEnd;
    %re-initialize
    x0 = x(end,:)'; %update intial condition
    x0(1) = x0(1) + dose;%add dose
    Dose = Dose +1;
    actualtime = t(end); %update 'intial' time
end

figure(1)
%plot desired concentration range
area([0 Simtime/((60*24))],[5e-4 5e-4],3.3e-4,'FaceColor',[.7 .7 .7],'EdgeColor','none')
hold on
%plot trajectory
h = plot(tOutput./(60*24),xOutput(:,2));
%legends/labels
xlabel('time [days]')
ylabel('concentrations [mol/L]')
set(gca,'YScale','log','Ylim', [0.0001 0.001])%plot y-axis logarithmically
legend(h,'central cmp')
PlotNicely(1)
%plot dosing events as vertical lines
for i = 1:floor(Simtime/DosingInterval)
    line([i*DosingInterval i*DosingInterval]./(60*24),[0.0001 0.001],'LineStyle','--','Color','k')
end

figure(2)
plot(tOutput./(60*24),xOutput(:,[1 3]).*repmat(MW([1 3])',length(tOutput),1))
xlabel('time [days]')
ylabel('[mg]')
set(gca,'YScale','log')%plot y-axis logarithmically
legend('dosing cmp.','sink')
PlotNicely(2)

figure(4)
plot(tOutput./(60*24),(xOutput*V).*Na) % sum over # mole sum_i ([mole_i/L] * [L]) - > [mole]
xlabel('time [days]')
ylabel('total # molecules in system')
PlotNicely(4)

figure(5)
stairs(tOutput./(60*24),xOutput*(V.*MW))
xlabel('time [days]')
ylabel('total mass in system [g]')
PlotNicely(5)